#################################################################################
# Created       : Martin Foster
# Modified      : 02/26/2002
#################################################################################
#
# Pull -  Post retrieval system
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Pull;

use Compress::Zlib;						# ZLib compression library
use MIME::Entity;						# MIME type handling

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Posts
#
# Allows a host to retreive the posts for a room.  This will fetch all of the
# posts currently in the database, then output the file using a normal GNU
# ZIP format.

sub Posts
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $choice   = shift;					# Choice of the matter

	my $date     = new Ethereal::Date();			# Date handler
	my $data     = $database->{'HANDLE'};			# Database handle
	my $statement;						# Database statement

	my $gz;							# Compression file handle
	my $mail;						# Mailed message
	my $message;						# Message to append
	my $post;						# Retreived post
	my $seperator;						# Seperator bar
	my $timestamp;						# Retreived date

	my %system;						# System hash


	####################
	# Program Area

	# Make sure output is necessary
	if ((defined($param->{'GETPOSTS'})) && (defined($param->{'ROOM'})))
	{
		# Assign timestamp
		$timestamp = $date->GetDateShort(time);

		# Retreive hash
		$database->GetHashSystem(\%system);


		# Retreive Email details
		($mail)      = $database->DataGetPuppeteerEmail($param->{'USER'});
		($message)   = $database->DocumentGetMailPosts();

		# Retreive seperator bar
		($seperator) = $database->DataGetRealmSeperator($param->{'ROOM'});
		($seperator) = (defined($seperator)) ? $seperator : $realm{'TagSeperator'};



		# Administrative and supervisory
		if (defined($choice))
		{
			# Is user an admin
			my ($admin) = $database->DataGetAdmin($param->{'USER'});

			# Prepare
			if ($admin eq 'yes')
			{
				# Administrators
				$statement = $data->prepare("SELECT PostPartialFormat
					FROM  Post
					WHERE RealmName=?
					  AND PostTo != ?
					ORDER BY PostIDNumber");
				$statement->execute($param->{'ROOM'}, 'system');
			}
			else
			{
				# Supervisors
				$statement = $data->prepare("SELECT PostPartialFormat
					FROM  Post
					WHERE RealmName=?
					  AND PostTo=?
					ORDER BY PostIDNumber");
				$statement->execute($param->{'ROOM'}, 'all');
			}
		}

		# Normal post retreival
		else
		{
			# Return if root undefined
			unless (defined($param->{'ROOM'}))
			{ return 0; }


			# Format handling
			# Retreive formatting used
			my ($formatting) = $database->DataGetFormatting($param->{'USER'});

			# Determin field to use
			# Full formatting
			if ($formatting eq 'complete')
			{ $formatting = 'PostFullFormat'; }

			# Full less images
			elsif ($formatting eq 'imageless')
			{ $formatting = 'PostImagelessFormat'; }

			# Spartan formatting
			elsif ($formatting eq 'spartan')
			{ $formatting = 'PostPartialFormat'; }


			# Define query
			$statement = $data->prepare("SELECT $formatting
				FROM  Post
				WHERE RealmName=?
  				  AND PostIDNumber > ?
  				  AND (PostTo=?
				   OR PostTo=?)
				ORDER BY PostIDNumber");

			
			# Execute statement
			$statement->execute($param->{'ROOM'}, $param->{'ROOT'}, $param->{'USER'}, 'all');
		}

		# Variables
		my $time = time;
		my $room = "$param->{ROOM} $param->{USER}";

		# Replaces spaces with underscores
		$room =~ s/\W+/_/gs;

		# Other replacements
		$message =~ s/REALM/$param->{'ROOM'}/gs;
		$message =~ s/DATE/$timestamp/gs;


		# Open filehandle
		$gz = gzopen("$system{LocTemp}/${room}_${time}.html.gz", "wb")
		 or die "Cannot open file: $!\n";
    

		# Display
		# HTML header 
		$gz->gzwrite($database->FetchRealmHeader($room)); 

		# Cycle through posts
		while ($post = $statement->fetchrow())
		{
			# Add into stream
			$gz->gzwrite($post); 
			$gz->gzwrite($seperator);
		}

		# HTML footer
		$gz->gzwrite($database->FetchRealmFooter($room));

		# Close
		$statement->finish();
		$gz->gzclose();


		# MIME Handling
		# Mail headers:
		my $top = MIME::Entity->build(
			Type     => "multipart/mixed",
			From     => "$system{SetInfoContactAddress}",
			To       => "$mail",
			Subject  => "Post extract for $param->{ROOM}");

		# Useless message
		$top->attach(Data=> $message);

		# Insert posts
		$top->attach(
			Path     => "$system{LocTemp}/${room}_${time}.html.gz",
			Type     => "application/x-gzip",
			Encoding => "base64");


		# Sendmail
		# Open handle
		open(SENDMAIL, "|$system{LocSendmail} -oi -t")
			or die("Cannot fork sendmail: $!");

		# Write contents
		$top->print(\*SENDMAIL);

		# Close handle
		close SENDMAIL;

		# Unlink
		unlink("$system{LocTemp}/${room}_${time}.html.gz");

		
		# Cleanup
		$param->Cleanup('GETPOSTS');
	}
}

# Alias for administrative posts
sub AdminPosts
{ Posts(shift, shift, shift, shift, 1); }

1;								# Return True
